from emva1288.report.report import info_setup
from emva1288.report.report import info_basic
from emva1288.report.report import info_marketing
from emva1288.report.report import info_op
from emva1288.report.report import Report1288
